/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.datatypes.SmoothedValue;
import eos.framework.*;
import eos.models.*;
import eos.models.Baseline2Model.Baseline2Farm;

/**
 * Uses a supply curve system to dynamically adjust the price of food.
 * 
 * @author Michael Adelson
 */
public class BL2DynamicFarm extends Baseline2Farm {

    /**
     * @param economy
     * @param goods
     */
    public BL2DynamicFarm(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        this.laborPrice = new SmoothedValue(0, ALPHA);
        this.foodPrice = new SmoothedValue(0, ALPHA);
    }

    /**
     * Exponential smoothing coefficient
     */
    public static double ALPHA = 0.1;
    
    /**
     * Smoothed value estimates for labor and food.
     */
    private final SmoothedValue laborPrice, foodPrice;

    /**
     * Weight put on the old price.
     */
    private static final double OLD_PRICE_WEIGHT = 0.85;

    /*
     * (non-Javadoc)
     * 
     * @see eos.models.Baseline2Model.Baseline2Farm#baseLine2FarmSubclassAct()
     */
    protected void baseLine2FarmSubclassAct() {
        // if the Markets have not been initialized, bid at historical prices.
        if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0) {
            // buy labor
            this.laborMarket.addBuyOffer(this, Baseline2Model.MAX_PRODUCTION
                    / Baseline2Model.LABOR_PRODUCTIVITY,
                    Baseline2Model.HISTORICAL_LABOR_PRICE);
            // sell food
            if (this.food.getQuantity() > 0.0)
                this.foodMarket.addSellOffer(this, this.food.getQuantity(),
                        Baseline2Model.HISTORICAL_FOOD_PRICE);
            // setup smoothed values
            this.laborPrice.setValue(Baseline2Model.HISTORICAL_LABOR_PRICE);
            this.foodPrice.setValue(Baseline2Model.HISTORICAL_FOOD_PRICE);
            return;
        }

        // otherwise, use normal bidding algorithms
        this.laborPrice.update(this.laborMarket.getLastMarketPrice());
        this.foodPrice.update(this.foodMarket.getLastMarketPrice());

        // buy labor using a demand curve method. Assumes a linear production
        // function
        double maxPrice = this.foodPrice.getValue()
                * Baseline2Model.LABOR_PRODUCTIVITY;
        double maxLabor = Baseline2Model.MAX_PRODUCTION
                / Baseline2Model.LABOR_PRODUCTIVITY;
        double m = (this.foodPrice.getValue() - maxPrice) / maxLabor;
        for (double x = 0.0; x < maxLabor; x += Laborer.LABOR_ALLOWANCE) {
            double price = m * x + maxPrice;
            // avg with the actual price
            // (price + this.laborPrice.getValue()) / 2.0;
            price = this.laborPrice.getValue() * OLD_PRICE_WEIGHT + price
                    * (1 - OLD_PRICE_WEIGHT);
            if (!this.laborMarket.addBuyOffer(this, Laborer.LABOR_ALLOWANCE,
                    price))
                break;
        }

        // final double TARGET_FOOD_STOCK = 3.0 * Baseline2Model.MAX_PRODUCTION;
        // // // 10 bids below
        // // //if (this.getl)
        // double qLabor = (TARGET_FOOD_STOCK - food.getQuantity())
        // Baseline2Model.LABOR_PRODUCTIVITY;
        // if (qLabor > 0.0)
        // this.laborMarket.addBuyOffer(this, qLabor, this.laborPrice
        // .getValue());

        // sell food using supply curve method assumes a linear production
        // function
        double quantity = this.food.getQuantity();
        double minPrice = this.laborPrice.getValue()
                / Baseline2Model.LABOR_PRODUCTIVITY;
        m = (this.laborPrice.getValue() - minPrice) / quantity;
        double step = Math.max(Laborer.EAT_AMOUNT, quantity / 20.0);
        for (double x = 0.0; x < quantity; x += step) {
            double price = minPrice + m * x;
            // take avg of this price and last price
            price = (price + this.foodPrice.getValue()) / 2.0;
            // if (this.economy.getRunningTimeSteps() <= 10)
            // System.out.println("Step = "
            // + this.economy.getRunningTimeSteps()
            // + ", Farm offering: Price " + price);
            if (!this.foodMarket.addSellOffer(this, step, price))
                break;
        }
    }
}
