/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.datatypes.SmoothedValue;
import eos.framework.*;
import eos.models.*;
import eos.models.Baseline2Model.Baseline2Laborer;

/**
 * Uses a demand curve system to dynamically adjust the price of food.
 * 
 * @author Michael Adelson
 */
public class BL2DynamicOL extends Baseline2Laborer {

    /**
     * @param economy
     * @param goods
     */
    public BL2DynamicOL(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        this.foodPrice = new SmoothedValue(0, BL2DynamicFarm.ALPHA);
        this.laborPrice = new SmoothedValue(0, BL2DynamicFarm.ALPHA);
    }

    /**
     * A comfortable supply of food.
     */
    private static final double TARGET_FOOD_STOCK = 30.0;

    /**
     * Smoothed value estimates for food, labor.
     */
    private final SmoothedValue foodPrice, laborPrice;

    /**
     * Weight put on the old price.
     */
    private static final double OLD_PRICE_WEIGHT = 0.85;

    /*
     * (non-Javadoc)
     * 
     * @see
     * eos.models.Baseline2Model.Baseline2Laborer#baseLine2LaborerSubclassAct()
     */
    protected void baseLine2LaborerSubclassAct() {
        
        // if the Markets have not been initialized, bid at historical prices.
        if (this.foodMarket.getLastMarketPrice() <= 0.0
                || this.laborMarket.getLastMarketPrice() <= 0.0) {
            // sell labor
            this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE,
                    Baseline2Model.HISTORICAL_LABOR_PRICE);
            // buy food
            this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                    Baseline2Model.HISTORICAL_FOOD_PRICE);
            // setup smoothed values
            this.laborPrice.setValue(Baseline2Model.HISTORICAL_LABOR_PRICE);
            this.foodPrice.setValue(Baseline2Model.HISTORICAL_FOOD_PRICE);
            return;
        }

        // otherwise use normal bidding algorithms
        // update smoothed estimates
        this.foodPrice.update(this.foodMarket.getLastMarketPrice());
        this.laborPrice.update(this.laborMarket.getLastMarketPrice());

        // buy food using a demand-curve strategy
        double b = this.laborMarket.getLastMarketPrice();
        double m = -(b - this.foodMarket.getLastMarketPrice())
                / TARGET_FOOD_STOCK;

        /*
         * PANIC Mode! If we have too little money to make the max bid, instead
         * bid all money at once. And sell labor at food price. The hope is that
         * we will be guaranteed to buy/ be hired, but that we won't actually
         * pay/receive these prices. This will thus help us recover.
         */
        if ((m * this.food.getQuantity() + b) * Laborer.EAT_AMOUNT > this.money
                .getQuantity()) {
            // System.err.println("#" + this.getID() + " panicking at step "
            // + this.economy.getRunningTimeSteps() + " m="
            // + this.money.getQuantity() + " fp="
            // + this.foodPrice.getValue() + " lp="
            // + this.laborPrice.getValue());
            // buy food
            if (this.money.getQuantity() > 0.0)
                this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT,
                        this.money.getQuantity());
            // sell labor
            this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE,
                    this.foodPrice.getValue());

            return;
        }

        // continue with the demand curve strategy
        for (double x = this.food.getQuantity(); x <= 1.1 * TARGET_FOOD_STOCK; x += Laborer.EAT_AMOUNT) {
            double price = m * x + b;
            if (price <= 0.0)
                break;
            // if (this.economy.getRunningTimeSteps() <= 10)
            // System.out.println("#" + this.getID() + " Adding buy offer: "
            // + price + " for x=" + x + " (I have "
            // + this.food.getQuantity() + ")");
            if (!this.foodMarket.addBuyOffer(this, Laborer.EAT_AMOUNT, price))
                break;
        }

        // sell labor using a demand curve method. Assumes a linear production
        // function
         double maxPrice = this.foodPrice.getValue()
         * Baseline2Model.LABOR_PRODUCTIVITY;
         m = (maxPrice - this.foodPrice.getValue()) / Laborer.LABOR_ALLOWANCE;
         double step = Laborer.LABOR_ALLOWANCE / 5.0;
         for (double x = 0.0; x < Laborer.LABOR_ALLOWANCE; x += step) {
         double price = maxPrice - m * x;
         // avg with the actual price
         // price = (price + this.laborPrice.getValue()) / 2.0;
         price = this.laborPrice.getValue() * OLD_PRICE_WEIGHT + price
         * (1 - OLD_PRICE_WEIGHT);
         if (!this.laborMarket.addSellOffer(this, step, price))
         break;
         }

//        double percentage = 0.1, price = this.laborPrice.getValue(), threshold = 0.75 * TARGET_FOOD_STOCK;
//        if (this.food.getQuantity() > TARGET_FOOD_STOCK)
//            price += percentage * this.laborPrice.getValue();
//        else if (this.food.getQuantity() < 0.75 * TARGET_FOOD_STOCK)
//            price -= ((threshold - this.food.getQuantity()) / threshold)
//                    * percentage;
//        if (price > 0.0)
//            this.laborMarket.addSellOffer(this, Laborer.LABOR_ALLOWANCE, price);
    }

}
