/**
 * 
 */
package eos.agents;

import java.util.Map;

import eos.framework.Economy;
import eos.framework.Good;
import eos.models.Baseline2Model;
import eos.models.Laborer;
import eos.models.Baseline2Model.Baseline2Farm;

/**
 * @author Michael Adelson
 * 
 */
public class BL2SeekingFarm extends Baseline2Farm {

    /**
     * The percent of the current price to adjust by.
     */
    public static final double ADJUST_PERCENT = 0.01;

    /**
     * Buy/Sell at lastPrice + priceAdjustment.
     */
    private double foodPriceAdjustment = 0, laborPriceAdjustment = 0;

    /**
     * Desired amount of labor. Used to calculate the percentage of labor
     * successfully bought.
     */
    private double laborBuyQuantity = Double.NaN;

    /**
     * The maximum amount of labor ever purchased. Used to estimate the actual
     * labor supply.
     */
    private double maxLaborBought = Laborer.LABOR_ALLOWANCE;

    /**
     * @see Baseline2Farm#Baseline2Farm(Economy, Map)
     */
    public BL2SeekingFarm(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
    }

    /**
     * Bids on Food and Labor.
     * 
     * @see eos.models.Baseline2Model.Baseline2Farm#baseLine2FarmSubclassAct()
     */
    protected void baseLine2FarmSubclassAct() {
        if (this.money.getQuantity() < 10.0)
            System.out.println("PANIC! Step="
                    + this.economy.getRunningTimeSteps() + " Money="
                    + this.money.getQuantity() + " Food="
                    + this.food.getQuantity());
        this.bidOnFood();
        this.bidOnLabor();
    }

    /**
     * Sell Food.
     */
    private void bidOnFood() {
        // nothing to sell
        if (this.food.getQuantity() <= 0.0)
            return;

        // initially just try to sell at historical price
        if (this.foodMarket.getLastMarketPrice() <= 0.0) {
            this.foodMarket.addSellOffer(this, this.food.getQuantity(),
                    Baseline2Model.HISTORICAL_FOOD_PRICE);
            return;
        }

        // basically sold all food, so increase prices with some probability
        if (this.food.getQuantity() - this.getLastAmountProduced() < Laborer.EAT_AMOUNT) {
            if (this.economy.random() < 0.1)
                this.foodPriceAdjustment += ADJUST_PERCENT
                        * this.foodMarket.getLastMarketPrice();
        } else
            // didn't sell all food, so try lowering prices
            this.foodPriceAdjustment -= ADJUST_PERCENT
                    * this.foodMarket.getLastMarketPrice();

        // sell all food at lastMarketPrice + foodPriceAdjustment
        double price = this.foodMarket.getLastMarketPrice()
                + this.foodPriceAdjustment;

        System.out.println("Step=" + this.economy.getRunningTimeSteps()
                + " fPrice=" + price + " adjust=" + this.foodPriceAdjustment);

        if (price > 0.0)
            this.foodMarket.addSellOffer(this, this.food.getQuantity(), price);
    }

    /**
     * Buy Labor
     */
    private void bidOnLabor() {
        // can't buy anything
        if (this.money.getQuantity() <= 0.0)
            return;

        // initially just try to buy at historical price
        if (this.laborMarket.getLastMarketPrice() <= 0.0) {
            this.laborBuyQuantity = Baseline2Model.MAX_PRODUCTION
                    / Baseline2Model.LABOR_PRODUCTIVITY;
            this.laborMarket.addBuyOffer(this, this.laborBuyQuantity,
                    Baseline2Model.HISTORICAL_LABOR_PRICE);
            return;
        }

        // bought most of the desired labor, so lower prices
        if (this.laborBuyQuantity - this.getLastStartingLabor() < Laborer.LABOR_ALLOWANCE) {
            if (this.economy.random() < 0.1) // lower prices with some
                // probability
                this.laborPriceAdjustment -= ADJUST_PERCENT
                        * this.laborMarket.getLastMarketPrice();
        } else
            // didn't buy all labor, so try increasing prices
            this.laborPriceAdjustment += ADJUST_PERCENT
                    * this.laborMarket.getLastMarketPrice();

        // buy labor while there is marginal productivity
        double price = this.laborMarket.getLastMarketPrice()
                + this.laborPriceAdjustment;

        System.out.println("Step=" + this.economy.getRunningTimeSteps()
                + " Price=" + price + " mp="
                + this.laborMarket.getLastMarketPrice() + " adj="
                + this.laborPriceAdjustment);
        // check that price is valid
        if (price <= 0.0)
            return;
        int laborUnits = 1;
        while (true) {
            double marginalProductivity = Baseline2Model
                    .convertToFood(laborUnits)
                    - Baseline2Model.convertToFood(laborUnits - 1);
            // don't bother producing if productivity is < half a meal
            if (marginalProductivity < (Laborer.EAT_AMOUNT / 2.0))
                break;
            // set an upper bound on the final price
            double finalPrice = Math.min(price, marginalProductivity
                    * this.foodMarket.getLastMarketPrice());
            this.laborMarket.addBuyOffer(this, 1, finalPrice);
            laborUnits++;
        }

        // update maxLaborBought
        this.maxLaborBought = Math.max(this.getLastStartingLabor(),
                this.maxLaborBought);
        // this.maxLaborBought = Double.POSITIVE_INFINITY;
        // update laborBuyQuantity, using maxLaborBought as an upper bound
        this.laborBuyQuantity = Math.min(laborUnits - 1, this.maxLaborBought);
        System.out.println("Step=" + this.economy.getRunningTimeSteps()
                + " BQ=" + this.laborBuyQuantity);
    }
}
