package eos.agents;

import java.util.Map;

import eos.models.BaselineModel.*;
import eos.framework.*;

/**
 * An extremely simple implementation of a day-trader; SimpleTrader extends
 * Laborer but simply does not sell any of his labor. The SimpleTrader buys food
 * for himself using the same technique as SimpleLaborer2. He then uses the rest
 * of his money to buy low and sell high based on an exponentially smoothed
 * value for food.
 * 
 * Author: Michael Adelson
 */
public class SimpleTrader extends BaselineLaborer {

    /**
     * When food quantity is below the threshold the laborer bids more than the
     * market price and vice versa.
     */
    private static final double FOOD_THRESHOLD = 30;

    /**
     * Constant used in exponential smoothing.
     */
    private static final double ALPHA = 0.1;

    /**
     * The economy's food and labor for money markets.
     */
    private Market foodMarket;

    /**
     * An exponentially smoothed estimate of the value of food.
     */
    private double value;

    /**
     * Create a new SimpleTrader
     */
    public SimpleTrader(Economy economy, Map<String, Good> goods) {
        super(economy, goods);
        foodMarket = null;
    }

    /**
     * The trader buys food for himself using the same method as SimpleLaborer.
     * He then uses the rest of his money/food to buy low and sell high.
     */
    protected void bid() {
        // update value of food
        if (foodMarket == null) { // initialize
            foodMarket = economy.getMarketFor(money.getName(), food.getName());
            value = this.economy.random() + 1e-3;
        } else
            value = ALPHA * foodMarket.getLastMarketPrice() + (1 - ALPHA)
                    * value;

        // bid for self using SimpleLaboerer's technique
        if (foodMarket.getLastMarketPrice() <= 0.0) {
            foodMarket.addBuyOffer(this, EAT_AMOUNT, this.economy.random() + 0.05);
            foodMarket.addBuyOffer(this, EAT_AMOUNT, this.economy.random() + 0.05);
            return;
        }

        double quantity = 2.0 * EAT_AMOUNT;
        double price = foodMarket.getLastMarketPrice()
                * (FOOD_THRESHOLD / (food.getQuantity() + 0.1));
        if (price * quantity > money.getQuantity())
            price = money.getQuantity() / quantity;

        if (price > 0.0)
            foodMarket.addBuyOffer(this, quantity, price);

        // buy low
        price = 0.95 * value;
        quantity = money.getQuantity() / price;
        if (quantity > 0.0)
            foodMarket.addBuyOffer(this, quantity, price);

        // sell high
        price = 1.25 * value;
        quantity = food.getQuantity() - FOOD_THRESHOLD;
        if (quantity > 0.0)
            foodMarket.addSellOffer(this, quantity, price);
    }

}
