/**
 * edit 3-23-2009 Chris Rucinski
 * edit 3-24-2009 Chris Rucinski
 * 	added more information to the toString() method
 * edit 10-11-09 Michael Adelson: changed goods to map with Class, rather than
 * String, keys.
 * edit 10-28-09 Changed goods back to map names, Goods.
 */

package eos.framework;

import java.util.*;

/**
 * The Agent is EOS's decision-making unit. An Economy may contain a 
 * variety of different types of Agents, from workers to firms. The abstract
 * Agent class contains the most basic Agent fields and methods: those which 
 * will be called by Economy. A single call to the act method asks the Agent
 * to perform 1 time-unit's worth of behavior.
 */
public abstract class Agent {
    
    /**
     * The Agent constructor gives each Agent a unique ID number. This is done
     * by incrementing nextAvailableID each time an Agent is instantiated.
     */
    private static int nextAvailableID = 1;
    
    /**
     * The enclosing Economy. The agent should use it's access to the 
     * Economy object to get information about the current simulation,
     * such as what Markets are available.
     */
    protected final Economy economy;

    /**
     * A unique integer identifying this Agent.
     */
    private final int myID;
    
    /**
     * Is the Agent alive/in business?
     */
    private boolean isAlive;
    
    /**
     * Create a new Agent in economy. The goods map should map Strings to Names.
     * Agent does nothing with goods.
     */
    public Agent(Economy economy, Map<String, Good> goods) {
        this.economy = economy;
        isAlive = true;
        myID = nextAvailableID;
        nextAvailableID++;
    }
    
    /**
     * All Agents must implement an act method which is called by Economy
     * each time step.
     */
    abstract public void act();
    
    /**
     * Returns a double value representing this Agent's net worth.
     */
    abstract public double getNetWorth();
    
    /**
     * Returns a Collection view of this Agent's Goods.
     */
    abstract public Collection<Good> getGoods();
    
    /**
     * Returns this Agent's reference to a specific Good. The basic
     * implementation may be less efficient for many classes extending Agent,
     * as it performs a linear search of this.getGoods(). It serves as an 
     * example of correct implementation and error-throwing behavior and should
     * be overridden by extending classes.
     */
    public Good getGood(String goodName)
    {
        Collection<Good> goods = this.getGoods();
        for (Good g : goods)
            if (g.getName().equals(goodName))
                return g;
        throw new RuntimeException("This Agent does not have " + goodName);
    }
    
    /**
     * Returns this Agent's unique ID number. This method cannot be overwritten
     * by classes that extend Agent, as this would remove its meaning.
     */
    public final int getID() { return myID;}
    
    /**
     * Returns true if this Agent is "alive," based on the state of the isAlive
     * instance variable. This method cannot be overwritten by classes that 
     * extend Agent, as this would remove its meaning.
     */
    public final boolean isAlive() { return isAlive;}
    
    /**
     * sets isAlive to false, effectively killing the agent
     */
    protected void die(){ isAlive = false;}
    
    /**
     * Returns basic information about this agent in the form of a String. This
     * method should be overriden to provide better information if necessary.
     */
    public String toString() {
    	String output = "";
    	output += "Agent type: " + this.getClass().toString() + "\n";
    	output += "Agent number " + getID() + "\n";
    	output += "\tisAlive = " + isAlive() + "\n";
    	output += "\tnet worth = " + getNetWorth() + "\n";
    	for (Good g : this.getGoods()) {
        	output += "quantity of " + g.getName() + ": " + g.getQuantity() + "\n";
    	}
    	return output;    	
    }
    
}


