/**
 * edit 3-23-09 Michael Adelson: Added methods for adding Agents and Markets, 
 * as well as accessing Markets. Added a constructor and changed 
 * runningTimeSteps to protected.
 * edit 4-2-09 Michael Adelson: Added the addPrinter method.
 * edit 4-3-09 Michael Adelson: Added a frequency argument to addPrinter.
 * edit 10-11-09 Michael Adelson: Added centralized random number generation 
 * utilities, moved most Agent- and Market-adding methods to BuildableEconomy.
 * edit 10-28-09 Michael Adelson: Made constructor initialize economy.random.
 * edit 10-28-09 Michael Adelson: Changed getMarketFor() to use new named Goods
 * convention.
 */
package eos.framework;

import java.util.Random;

/**
 * The Economy class provides 4 functions:
 * 1. Contains references to all Agents in the simulation and manages the 
 * creation/death of Agents where applicable.
 * 2. Allows Agents to get global information about the simulation.
 * 3. Drives the simulation by calling each Agent's act method.
 * 4. Handles user input and output.
 * 
 * Note that classes extending Economy may also wish to implement the I/O 
 * interfaces DataOutputEconomy and BuildableEconomy.
 * 
 * @author Michael Adelson
 */
public abstract class Economy {

    /**
     * The number of steps that the simulation has run so far.
     */
    protected int runningTimeSteps;
    
    /**
     * Random number generator used to centralize all random number generation.
     * Setting the seed of random should ensure repeatable results; however, 
     * this may not be true if multi-threaded EOS implementations are used.
     */
    protected Random random; 
    
    /**
     * Create an empty Economy, initializing runningTimeSteps to 0.
     * 
     * Also initializes this.random to a new Random().
     */
    public Economy() {
        this.runningTimeSteps = 0;
        this.random = new Random();
    }
    
    /* Methods for running the simulation */
    
    /**
     * Run the simulation through 1 time-step, calling each Agent's act method
     * once. 
     */
    public abstract void step();
    
    /**
     * This method should be equivalent to calling step() steps times.
     */
    public abstract void run(int steps);
    
    /**
     * Get the number of steps the simulation has run.
     */
    public int getRunningTimeSteps() {
        return this.runningTimeSteps;
    }
    
    /**
     * Returns a reference to the Market in this Economy that uses currency
     * and good. This method should return NULL if no such Market exists.
     */
    public abstract Market getMarketFor(String currencyName, 
                                        String productName);
    
    /* 
     * Methods dealing with Random Number Generation 
     * These methods provide simple, efficient implementations. These
     * implementations should be overriden with more secure ones if more 
     * error checking is desired. 
     */
    
    /**
     * Get the random number generator for this Economy. The primary use of this
     * method is to allow simulation classes to get repeatable results by 
     * calling economy.getRandom.setSeed() to seed the random number generator.
     * More most random number generation, economy.random() should be easier to 
     * use. 
     */
    public Random getRandom() {
        return this.random;
    }
    
    /**
     * Get a random double in [0, 1) (Most callers will find using this
     * method easier than getRandom().
     */
    public double random() {
        return this.random.nextDouble();
    }  
    
    /*
     * Economy building methods. These methods allow new Agents and Markets to 
     * be added to the economy. More advanced methods are specified in 
     * BuildableEconomy.
     */
    
    /**
     * Add agent to the economy.
     */
    public abstract void addAgent(Agent agent);
    
    /**
     * Add market to the economy.
     */
    public abstract void addMarket(Market market);
}
