/**
 * edit Michael Adelson 3-29-09: Added the method isDiscrete
 * edit 10-28-09 Michael Adelson: Added getName() method.
 */
package eos.framework;

/**
 * Good defines a set of methods for objects that can be owned by Agents.
 * An instance of a Good holds some quantity of the good it represents. For 
 * example, a money object might use a private double field to represent its
 * quantity.
 * 
 */
public interface Good {

    /**
     * Returns the name of this Good. The name of a Good should represent the
     * class of commodity it is. For example: Food, Labor, and Money would be
     * appropriate names. getName() could return a constant string set by the
     * class (if commodity-specific methods were desired) or could return a 
     * name set by the constructor. Strategy-level implementations of Good are
     * responsible for checking that Goods are only instantiated with the 
     * appropriate names.
     **/
    public String getName();
    
    /**
     * Reduce this Good's quantity by amount. The return value should indicate
     * the amount successfully withdrawn. This allows calls that would
     * "overdraw" a Good to fail gracefully. Thus, for Goods Giver and Receiver, 
     * the statement: Receiver.deposit(Giver.withdraw(x)) should always result
     * in a) conservation of quantity and possibly b) an exception being thrown.
     * For inputs that are unacceptable under any circumstances (e.g. if amount
     * is negative), it is recommended that Good implementations do not adjust
     * their quantity and throw an exception.
     */
    public double withdraw(double amount);
    
    /**
     * Increase this Good's quantity by amount. Improper values for amount
     * (e.g. amount is negative) should cause the implementation to throw
     * an exception. What constitutes an improper value depends on the 
     * implementation.
     */
    public void deposit(double amount);
    
    /**
     * Returns a double representation of the quantity of this Good.
     */
    public double getQuantity();
    
    /**
     * Returns true if the Good can only be handled in discrete (integer) 
     * quantities, and false otherwise.
     */
    public boolean isDiscrete();    
}
