package eos.io;

import java.util.*;
import eos.framework.*;

/**
 * A dataPrinter for outputting market prices.
 * 
 * @author Michael Adelson
 */
public class MarketPriceDataPrinter implements DataPrinter {

    private final String currency, product;

    /**
     * The default currency name used.
     */
    public static String DEFAULT_CURRENCY = "Money";

    /**
     * Assumes currency is DEFAULT_CURRENCY.
     */
    public MarketPriceDataPrinter(String product) {
        this.product = product;
        this.currency = DEFAULT_CURRENCY;
    }

    /**
     * Create a MarketPriceDataPrinter for currency and product.
     */
    public MarketPriceDataPrinter(String currency, String product) {
        this.product = product;
        this.currency = currency;
    }

    /**
     * Calls this.getData(economy).
     */
    public String getCleanupData(DataOutputEconomy economy) {
        return this.getData(economy);
    }

    /**
     * Returns the last MarketPrice, or a "Not Found" statement.
     */
    public String getData(DataOutputEconomy economy) {
        Market m = economy.getMarketFor(currency, product);
        if (m == null)
            return "No Market Found";
        else
            return m.getLastMarketPrice() + "";
    }

    /*
     * (non-Javadoc)
     * 
     * @see output.DataPrinter#getTitle(output.DataOutputEconomy)
     */
    public String getTitle(DataOutputEconomy economy) {
        if (this.currency.equals(DEFAULT_CURRENCY))
            return this.product + " price";
        else
            return this.product + " price ("
                    + this.currency + ")";
    }

    /**
     * True.
     */
    public boolean isCSVCompatible() {
        return true;
    }

    /**
     * True.
     */
    public boolean isMultiRunCompatible() {
        return true;
    }

    /**
     * Create a MarketPriceDataPrinter for each Market in economy.
     */
    public static Collection<DataPrinter> getDataPrintersFor(
            DataOutputEconomy economy) {
        List<DataPrinter> list = new ArrayList<DataPrinter>();
        for (Market m : economy.getMarkets()) {
            if (m.getCurrency().equals(DEFAULT_CURRENCY))
                list.add(new MarketPriceDataPrinter(m.getProduct()));
            else
                list.add(new MarketPriceDataPrinter(m.getCurrency(), m
                        .getProduct()));
        }

        return list;
    }
}
