package eos.io;

import eos.utilities.*;
import eos.framework.Agent;
import java.util.*;

/**
 * DataPrinter for printing the number of Agents currently alive in the
 * simulation. NumberAliveDataPrinter only recounts Agents of a certain class
 * when the number of Agents changes. This permits much greater efficiency but
 * risks incorrect calculation if Agents are being dynamically added to the
 * simulation.
 * 
 * by Michael Adelson
 */
public class NumberAliveDataPrinter implements DataPrinter {

    // the class this DP is counting
    private final Class<? extends Agent> agentClass;

    // a map of lastNumberAlive values for multiRun printing
    private final Map<DataOutputEconomy, Integer> lastNumberAliveMap;

    // is this DP configured to be multiRunCompatible?
    private final boolean isMultiRunCompatible;

    // the last number of agents that were alive
    private int lastNumberAlive = 0;

    /**
     * This DataPrinter prints the total number of alive Agents. The DataPrinter
     * is multiRun compatible
     */
    public NumberAliveDataPrinter() {
        this(null, true);
    }

    /**
     * Construct a DataPrinter for a specific Agent class
     */
    public NumberAliveDataPrinter(Class<? extends Agent> agentClass,
            boolean isMultiRunCompatible) {
        this.agentClass = agentClass;
        this.isMultiRunCompatible = isMultiRunCompatible;

        if (isMultiRunCompatible)
            this.lastNumberAliveMap = new HashMap<DataOutputEconomy, Integer>();
        else
            this.lastNumberAliveMap = null;
    }

    /**
     * Frees reference to economy for the purpose of garbage collection, then
     * calls this.getData(economy).
     */
    public String getCleanupData(DataOutputEconomy economy) {
        String data = this.getData(economy);
        if (this.isMultiRunCompatible)
            this.lastNumberAliveMap.remove(economy);
        return data;
    }

    /**
     * Get the number of alive agents.
     */
    public String getData(DataOutputEconomy economy) {
        Collection<Agent> agents = economy.getAgents();
        int count = 0;

        if (this.isMultiRunCompatible) {
            // add the economy if it hasn't been seen before
            if (!this.lastNumberAliveMap.containsKey(economy))
                this.lastNumberAliveMap.put(economy, 0);
            this.lastNumberAlive = this.lastNumberAliveMap.get(economy);
        }

        if (this.agentClass == null)
            count = agents.size();
        else if (this.isMultiRunCompatible && agents.size() != lastNumberAlive) {
            for (Agent a : agents)
                if (this.agentClass.isAssignableFrom(a.getClass()))
                    count++;

            this.lastNumberAliveMap.put(economy, count);
        } else if (agents.size() != lastNumberAlive) {
            for (Agent a : agents)
                if (this.agentClass.isAssignableFrom(a.getClass()))
                    count++;

            this.lastNumberAlive = count;
        } else {
            count = this.lastNumberAlive;
        }

        return count + "";
    }

    /**
     * Returns a generic or class-specific title.
     */
    public String getTitle(DataOutputEconomy economy) {
        if (this.agentClass == null)
            return "Living Agents";
        else
            return "Living " + Strings.getShortNames(this.agentClass)
                    + "s";
    }

    /**
     * True
     */
    public boolean isCSVCompatible() {
        return true;
    }

    /**
     * Dependent on the boolean passed to the constructor.
     */
    public boolean isMultiRunCompatible() {
        return this.isMultiRunCompatible;
    }

    /**
     * Returns a Collection of DataPrinters which represent the classes of
     * Agents currently in economy. Also returns a generic
     * NumberAliveDataPrinter.
     */
    public static Collection<DataPrinter> getDataPrintersFor(
            DataOutputEconomy economy, boolean areMultiRunCompatible) {
        List<DataPrinter> list = new ArrayList<DataPrinter>();
        Set<Class<? extends Agent>> classes = new HashSet<Class<? extends Agent>>();

        for (Agent a : economy.getAgents())
            classes.add(a.getClass());

        list.add(new NumberAliveDataPrinter()); // generic printer
        for (Class<? extends Agent> c : classes)
            list.add(new NumberAliveDataPrinter(c, areMultiRunCompatible));

        return list;
    }
}
