/** 
 * edit 3-24-09 Michael Adelson: fixed corner case where one either buyOffers or
 * sellOffers was empty.
 * 
 * edit 3-25-09 Michael Adelson: fixed corner case where p == prices.size().
 * 
 * edit 3-29-09 Michael Adelson: added code to addBuyOffer to support 
 * discrete products. Further code should be added to transact if we desire to 
 * support discrete currencies. Also, changed the checks on input price and 
 * maxQuantities to better handle 0 offers.
 * 
 * edit 4-1-09 Michael Adelson: fixed >= checks in the addOffer methods so that
 * they are more stringent and produce more useful error messages.
 * 
 * edit 4-2-09 Michael Adelson: added check in transact to handle corner case
 * where offers are exhausted simultaneously. Also changed the code for
 * restoring the money set aside by addBuyOffer (restored in transact) to use
 * try/catch so as to avoid 0 deposits, which cause Good to throw an error.
 * 
 * edit 4-21-09 Michael Adelson: when the CAM is choosing between the two valid
 * prices in clear, it now discards a price if no transactions would occur at
 * that price. This makes the market more efficient in the case when all supply
 * or demand is at one price. Also, the buyOffers and sellOffers lists are
 * shuffled before being sorted; this assures that no Agents get a consistent
 * advantage when many offer the same price. 
 * 
 * edit 10-21-09 Michael Adelson: updated CAM to conform to new Market 
 * specification.
 * 
 * edit 12-17-09 Michael Adelson: made sure that CAM did true volume maximization,
 * changed some things to protected to allow inheritance.
 */
package eos.markets;

import eos.framework.*;
import java.util.*;

/**
 * CallAuctionMarket extends implements the abstract class Market to work in the
 * following way: 1) The Market is cleared by finding the intersection of the
 * "supply" and "demand" curves created by the sell and buy offers; the
 * intersection price is declared as the market price. The market then makes all
 * transactions possible, starting with the highest paying buyer and the lowest
 * priced seller. All transactions are made at the market price. The complexity
 * of the Market's clear method is O(NlgN), since the offers must be sorted. 2)
 * When offers are added to the Market, it sets aside the offered currency and
 * product until the Market is cleared. This prevents Agents from unduly
 * influencing Market outcomes by offering bids that they cannot honor.
 * 
 * @author Michael Adelson
 */

public class CallAuctionMarket extends Market {

    // lists of current offers
    protected ArrayList<Offer> buyOffers, sellOffers;

    // the last market price (set by clear) -- initially set to 0
    protected double marketPrice;

    // the last market volume
    protected double marketVolume;

    // used for randomization
    protected final Economy economy;

    /**
     * Protected class representing an offer to buy or sell. Offers are
     * comparable by price. Offers are used here simply as wrappers, and thus do
     * not perform any operations or type-checking on the data.
     */
    protected class Offer implements Comparable<Offer> {

        Good currency, product;
        double maxQuantity, price;
        String agentType;
        
        public String toString()
        { String retVal="("+this.agentType+" , "+this.price+" , "+this.maxQuantity+")"; return retVal;}

        /**
         * Compare this Offer to that by price.
         */
        public int compareTo(Offer that) {
            if (this.price > that.price)
                return 1;
            else if (this.price < that.price)
                return -1;
            return 0;
        }
    }

    /**
     * Construct a Market using Currency and Product. Stores economy for use in
     * randomization.
     */
    public CallAuctionMarket(Economy economy, String currencyName,
            String productName) {
        super(economy, currencyName, productName);
        buyOffers = new ArrayList<Offer>();
        sellOffers = new ArrayList<Offer>();
        marketPrice = 0;
        marketVolume = 0;
        this.economy = economy;
    }

    /**
     * Add an offer to buy up to maxQuantity of Product at price (in currency).
     * 
     * This implementation of addBuyOffer will throw RuntimeExceptions if the
     * input Agent lacks Currency or Product goods, or if either price or
     * maxQuantity are <= 0. It generates an offer for the real maximum
     * quantity, i.e. the maximum quantity that could be purchased at price
     * given available currency. This amount is also "set aside" to prevent
     * Agents from making bids that they cannot honor. The method only returns
     * false if its withdraw call to the currency Good throws an exception.
     */
    public boolean addBuyOffer(Agent buyer, double maxQuantity, double price) {
        Good currency = buyer.getGood(this.currencyName);
        Good product = buyer.getGood(this.productName);
        return this.addBuyOffer(currency, product, maxQuantity, price, buyer.getClass().toString());
    }

    /**
     * Add an offer to sell up to maxQuantity of Product at price (in currency).
     * 
     * This implementation of addBuyOffer will throw RuntimeExceptions if the
     * input Agent lacks Currency or Product goods, or if either price or
     * maxQuantity are <= 0. It generates an offer for the real maximum
     * quantity, i.e. the maximum quantity that could be purchased at price
     * given available currency. This amount is also "set aside" to prevent
     * Agents from making bids that they cannot honor. The method only returns
     * false if its withdraw call to the currency Good throws an exception.
     */
    public boolean addSellOffer(Agent seller, double maxQuantity, double price) {
        Good currency = seller.getGood(this.currencyName);
        Good product = seller.getGood(this.productName);
        return this.addSellOffer(currency, product, maxQuantity, price, seller.getClass().toString());
    }

    /**
     * Add an offer to buy up to maxQuantity of product at price (in currency).
     * 
     * This implementation of addBuyOffer will throw RuntimeExceptions if the
     * input Good's types are not compatible with currency and product, or if
     * either price or maxQuantity are <= 0. It generates an offer for the real
     * maximum quantity, i.e. the maximum quantity that could be purchased at
     * price given available currency. This amount is also "set aside" to
     * prevent Agents from making bids that they cannot honor. The method only
     * returns false if its withdraw call to the currency Good throws an
     * exception.
     */
    public boolean addBuyOffer(Good currency, Good product, double maxQuantity,
            double price, String buyName) {
        // check types
        if (!currency.getName().equals(this.currencyName))
            throw new RuntimeException("Currency type mismatch");
        if (!product.getName().equals(this.productName))
            throw new RuntimeException("Product type mismatch");
        if (maxQuantity <= 0)
            throw new RuntimeException(maxQuantity + " is not an allowed value"
                    + " for maxQuantity: must be > 0");
        if (price <= 0)
            throw new RuntimeException(price + " is not an allowed value"
                    + " for price: must be > 0");

        // "set aside" maximum currency needed and
        // calculate the feasible maximum quantity based on available currency
        double realMaxQuantity = 0;
        try {
            realMaxQuantity = currency.withdraw(maxQuantity * price) / price;
        } catch (Exception e) {
            return false;
        }
        // check that realMaxQuantity is an allowed value for product
        if (product.isDiscrete()) { // cast realMaxQuantity to an integer
            double frQuantity = realMaxQuantity;
            realMaxQuantity = (int) realMaxQuantity;
            // deposit remainder
            currency.deposit((frQuantity - realMaxQuantity) * price);
        }

        // create and store the offer
        Offer offer = new Offer();
        offer.currency = currency;
        offer.product = product;
        offer.maxQuantity = realMaxQuantity;
        offer.price = price;
        offer.agentType = buyName;
        buyOffers.add(offer);

        return true;
    }

    /**
     * Add an offer to sell up to maxQuantity of product at price (in currency).
     * 
     * This implementation of addSellOffer will throw RuntimeExceptions if the
     * input Good's types are not compatible with currency and product, or if
     * either price or maxQuantity are <= 0. It generates an offer for the real
     * maximum quantity, i.e. the maximum quantity that could be sold at given
     * the available product. This amount is also "set aside" in order to
     * prevent Agents from making bids that they cannot honor. The method only
     * returns false if its withdraw call to the product Good throws an
     * exception.
     */
    public boolean addSellOffer(Good currency, Good product,
            double maxQuantity, double price, String selName) {
        // check types
        if (!currency.getName().equals(this.currencyName))
            throw new RuntimeException("Currency type mismatch");
        if (!product.getName().equals(this.productName))
            throw new RuntimeException("Product type mismatch");
        if (maxQuantity <= 0)
            throw new RuntimeException(maxQuantity + " is not an allowed value"
                    + " for maxQuantity: must be > 0");
        if (price <= 0)
            throw new RuntimeException(price + " is not an allowed value"
                    + " for price: must be > 0");

        // "set aside" maximum quantity needed
        // calculate the feasible maximum quantity based on available product
        double realMaxQuantity = 0;
        try {
            realMaxQuantity = product.withdraw(maxQuantity);
        } catch (Exception e) {
            return false;
        }

        // create and store the offer
        Offer offer = new Offer();
        offer.currency = currency;
        offer.product = product;
        offer.maxQuantity = realMaxQuantity;
        offer.price = price;
        offer.agentType = selName;
        sellOffers.add(offer);

        return true;
    }

    /**
     * Clears the market. First, the market price is determined by finding the
     * price at which the quantity that would be sold = quantity demanded. Then,
     * orders are filled starting with the highest buyOffer and and the lowest
     * sellOffer. All Goods sold are sold at exactly the market price.
     */
    public void clear() {
        // reset market volume
        this.marketVolume = 0.0;

        // sort Offers
        Collections.shuffle(buyOffers, this.economy.getRandom());
        Collections.shuffle(sellOffers, this.economy.getRandom());
        Collections.sort(buyOffers);
        Collections.sort(sellOffers);

        // if (this.economy.getRunningTimeSteps() == 2
        // && this.productName.equals("Labor")) {
        // System.out.println("SellOffers:");
        // for (Offer o : this.sellOffers)
        // System.out.print(o.price + ", ");
        // System.out.println("\n\rBuyOffers:");
        // for (Offer o : this.buyOffers)
        // System.out.print(o.price + ", ");
        // System.out.println();
        // }

        // handle special cases
        // highest buy price < lowest sell price -> no transactions OR
        // empty offer list -> no transactions
        if (buyOffers.isEmpty()
                || sellOffers.isEmpty()
                || less(buyOffers.get(buyOffers.size() - 1).price, sellOffers
                        .get(0).price)) {
            // return products to failed sellers
            for (Offer o : sellOffers)
                o.product.deposit(o.maxQuantity);
            // return currency to failed buyers
            for (Offer o : buyOffers)
                o.currency.deposit(o.maxQuantity * o.price);

            // reset offer lists
            buyOffers.clear();
            sellOffers.clear();
            return;
        }

        // merge prices into sorted list, eliminating duplicates
        ArrayList<Double> prices = new ArrayList<Double>(buyOffers.size()
                + sellOffers.size());
        int s = 0, b = 0;
        while (s < sellOffers.size() && b < buyOffers.size()) {
            if (equal(sellOffers.get(s).price, buyOffers.get(b).price)) {
                prices.add(sellOffers.get(s++).price);
                b++;
            } else if (sellOffers.get(s).price < buyOffers.get(b).price)
                prices.add(sellOffers.get(s++).price);
            else
                prices.add(buyOffers.get(b++).price);
            while (s < sellOffers.size() // eliminate duplicates in sellOffers
                    && equal(prices.get(prices.size() - 1),
                            sellOffers.get(s).price))
                s++;
            while (b < buyOffers.size() // eliminate duplicates in buyOffers
                    && equal(prices.get(prices.size() - 1),
                            buyOffers.get(b).price))
                b++;
        } // then add the remainder if there is one
        if (s < sellOffers.size())
            while (s < sellOffers.size()) {
                if (!equal(prices.get(prices.size() - 1),
                        sellOffers.get(s).price))
                    prices.add(sellOffers.get(s).price);
                s++;
            }
        else if (b < buyOffers.size())
            while (b < buyOffers.size()) {
                if (!equal(prices.get(prices.size() - 1),
                        buyOffers.get(b).price))
                    prices.add(buyOffers.get(b).price);
                b++;
            }
        /*AD edit
          System.out.println(buyOffers+"/nXXXXXXXXX/n");
                System.out.println(sellOffers+"/nSELLXXXXXXXXX/n");
         */       
         
//        if (this.economy.getRunningTimeSteps() == 50
//         && this.productName.equals("Food")) {
//         System.out.println("SellOffers:");
//         for (Offer o : this.sellOffers)
//         System.out.println(o);
//         System.out.println("\n\rBuyOffers:");
//         for (Offer o : this.buyOffers)
//         System.out.println(o);
//         }
//        if (this.economy.getRunningTimeSteps() == 50
//         && this.productName.equals("Labor")) {
//         System.out.println("LaborSellOffers:");
//         for (Offer o : this.sellOffers)
//         System.out.println(o);
//         System.out.println("\n\rLaborBuyOffers:");
//         for (Offer o : this.buyOffers)
//         System.out.println(o);
//         }
                
        
        
        
        // find the price(s) at which supply and demand intersect
        double supply = 0, demand = 0;
        // initialize demand to be the demand at lowest demand price
        for (Offer o : buyOffers)
            demand += o.maxQuantity;
        // walk through prices, updating supply and demand as needed.
        int p = 0;
        double bestVolume = Double.NEGATIVE_INFINITY;
        int bestP = 0;
        for (s = -1, b = 0; s + 1 < sellOffers.size() && b < buyOffers.size(); p++) {
            // if prices[p] is the next supply price:
            while (s + 1 < sellOffers.size()
                    && equal(prices.get(p), sellOffers.get(s + 1).price))
                supply += sellOffers.get(++s).maxQuantity;
            // if the current demand price is less than prices[p]:
            while (b < buyOffers.size()
                    && less(buyOffers.get(b).price, prices.get(p)))
                demand -= buyOffers.get(b++).maxQuantity;

            if (supply > demand) {
                // volume = demand
                if (demand > bestVolume) {
                    bestVolume = demand;
                    bestP = p;
                }
                break; // stop
            } else if (supply > bestVolume) { // volume = supply here
                bestVolume = supply;
                bestP = p;
            }
        }

        /*
         * If the above loop broke because the final supply price was reached,
         * then supply at Price >= prices[p-1] remains constant. Thus, it is
         * necessary to check whether demand at higher prices will ever become
         * less than the current supply. If so, then that will be at the market
         * price.
         */
        if (s + 1 == sellOffers.size()) {
            for (; b < buyOffers.size() && p < prices.size(); p++) {
                while (b < buyOffers.size()
                        && less(buyOffers.get(b).price, prices.get(p)))
                    demand -= buyOffers.get(b++).maxQuantity;

                if (supply > demand) {
                    // volume = demand
                    if (demand > bestVolume) {
                        bestVolume = demand;
                        bestP = p;
                    }
                    break; // stop
                } else if (supply > bestVolume) { // volume = supply here
                    bestVolume = supply;
                    bestP = p;
                }
            }
        }
        /*
         * If one of the loops broke because the next price was greater than the
         * highest demand price, then the market price is set to the highest
         * demand price in order to minimize shortage.
         */
        if (b == buyOffers.size())
            marketPrice = buyOffers.get(b - 1).price;
        /* If supply never crosses demand, set to highest supply price */
        else if (p == prices.size())
            marketPrice = prices.get(p - 1);
        /*
         * Finally, if the loop broke because supply > demand, there are two
         * valid prices that could be the market price: prices[p], which is the
         * smallest price at which supply tops demand, or prices[p-1], which is
         * the largest price at which demand tops supply. Since either price is
         * valid, we select between them in an arbitrary, yet deterministic
         * manner. However, if choosing between a two prices, one of which would
         * not cause any transactions to occur, we choose the other price.
         */
        else {
            if (p == 0)
                marketPrice = prices.get(p);
            else if (less(prices.get(p - 1), sellOffers.get(0).price))
                marketPrice = prices.get(p);
            else if (less(buyOffers.get(buyOffers.size() - 1).price, prices
                    .get(p)))
                marketPrice = prices.get(p - 1);
            else {
                marketPrice = prices.get(bestP);
            }
            // else { // random seed is deterministic
            // long r = Math.round(p + buyOffers.size() + sellOffers.size()
            // + supply + demand);
            //
            // if (r % 2 == 0)
            // marketPrice = prices.get(p);
            // else
            // marketPrice = prices.get(p - 1);
            //
            // }
        }

        // make all transactions possible at marketPrice
        int seller = 0, buyer = buyOffers.size() - 1;
        while (seller < sellOffers.size() && buyer >= 0
                && !less(marketPrice, sellOffers.get(seller).price) // sP <= mP
                && !less(buyOffers.get(buyer).price, marketPrice)) { // bP >= mP
            if (transact(sellOffers.get(seller), buyOffers.get(buyer),
                    marketPrice))
                seller++; // sell offer exhausted, move to next
            else
                buyer--; // buy offer exhausted, move to next
        }

        // return products to failed sellers
        Offer o;
        for (; seller < sellOffers.size(); seller++) {
            o = sellOffers.get(seller);
            o.product.deposit(o.maxQuantity);
        }

        // return currency to failed buyers
        for (; buyer >= 0; buyer--) {
            o = buyOffers.get(buyer);
            o.currency.deposit(o.maxQuantity * o.price);
        }

        // reset offer lists
        buyOffers.clear();
        sellOffers.clear();
    }

    /**
     * Returns the market price calculated by the last call to clear. Returns 0
     * if clear has not yet been called for this market.
     */
    public final double getLastMarketPrice() {
        return marketPrice;
    }

    /**
     * Returns the quantity of product transacted
     */
    public final double getLastMarketVolume() {
        return this.marketVolume;
    }

    /*---------------------------PRIVATE-METHODS---------------------------*/

    // epsilon variable used to compare doubles
    private static final double eps = 1e-5;

    /**
     * Private method used to compare double prices. If two prices differ by
     * less than eps, they are considered equal. This is necessary in order to
     * properly treat input double values that should be equal.
     */
    protected boolean equal(double a, double b) {
        if (a + eps < b)
            return false;
        if (a - eps > b)
            return false;
        return true;
    }

    /**
     * Private method used to compare double prices. a is less than b iff a is
     * less than b by more than eps. This is necessary in order to properly
     * treat input double values that should be equal.
     */
    protected boolean less(double a, double b) {
        if (a + eps < b)
            return true;
        return false;
    }

    /**
     * Sell as much as possible from sellOffer to buyOffer at salePrice. Returns
     * true if the sellOffer is exhausted, and false if the buyOffer is
     * exhausted (if both are exhausted returns false). Increments
     * this.marketVolume appropriately. Used in clear.
     */
    protected boolean transact(Offer sellOffer, Offer buyOffer, double salePrice) {
//        if (this.productName.equals("Food")) {
//            System.out.println(this.buyOffers.get(0).price + " " + this.sellOffers.get(0).price);
//        }
        
        // do not proceed at zero quantity
        if (sellOffer.maxQuantity <= 0)
            return true;
        if (buyOffer.maxQuantity <= 0)
            return false;

        // else, proceed with the transaction
        double quantity, cost;
        boolean returnVal = sellOffer.maxQuantity < buyOffer.maxQuantity;
        quantity = Math.min(sellOffer.maxQuantity, buyOffer.maxQuantity);
        cost = quantity * salePrice;

        // perform the exchange
        buyOffer.product.deposit(quantity);
        sellOffer.currency.deposit(cost);

        // if buyOffer.price > salePrice, restore set-aside currency
        try {
            buyOffer.currency.deposit((buyOffer.price - salePrice) * quantity);
        } catch (Exception e) { /* do nothing */
        }

        // reduce quantities for later transactions
        buyOffer.maxQuantity -= quantity;
        sellOffer.maxQuantity -= quantity;

        // record the quantity transacted
        this.marketVolume += quantity;

        return returnVal;
    }
}
