/**
 * 
 */
package eos.models;

import java.util.*;

import eos.framework.*;
import eos.utilities.Errors;

/**
 * A class defining the BaseLine2 Model. Contains the model level Agents and
 * Goods for BaseLine2.
 * 
 * This is a new way of defining an EOS model. Although it makes for a lengthy
 * java file, all related classes are grouped in one place. Alternatively, these
 * classes could be placed in a package of their own. As models get larger, that
 * method will probably become preferable. For now, though, this method provides
 * a simple way to encapsulate the essence of a model.
 * 
 * @author Michael Adelson
 */
public class Baseline2Model {

    /*------------------------------ Constants ------------------------------*/

    /**
     * The productivity of Labor in food production.
     */
    public static final double LABOR_PRODUCTIVITY = 2.5;

    /**
     * The maximum amount of food a farm can produce in one turn.
     */
    public static final double MAX_PRODUCTION = 60.0;

    /**
     * An arbitrary value representing the price of food before the simulation
     * began. Agents may wish to use this value as a basis for initial bidding.
     */
    public static double HISTORICAL_FOOD_PRICE = 0.2;

    /**
     * An arbitrary value representing the price of food before the simulation
     * began. Agents may wish to use this value as a basis for initial bidding.
     */
    public static double HISTORICAL_LABOR_PRICE = LABOR_PRODUCTIVITY
            * HISTORICAL_FOOD_PRICE * Laborer.EAT_AMOUNT
            / Laborer.LABOR_ALLOWANCE;

    /*----------------- BaseLine2 Goods ----------------------*/

    /**
     * The Goods used in BL2
     * 
     * @author Michael Adelson
     */
    public enum Goods {
        Food, Labor, Money, Leisure;
    }

    /**
     * Defines a fixed set of Goods for use in the BaseLine2 economy.
     * 
     * @author Michael Adelson
     */
    public static class Baseline2Good extends NameAndQuantityGood {

        /**
         * Register allowed Goods with superclass.
         */
        static {
            for (Goods goodName : Goods.values())
                NameAndQuantityGood.registerName(Baseline2Good.class, goodName
                        .toString());
        }

        /**
         * Construct a BaseLine2Good with name.
         */
        public Baseline2Good(String name) {
            super(name);
        }
    }

    /*----------------- BaseLine2 Agents ----------------------*/

    /**
     * Abstract class which sets up the Goods and Markets used in the Baseline2
     * economy.
     * 
     * @author Mike Adelson
     */
    public static abstract class Baseline2Laborer extends OwnerLaborer {

        // This Agent's goods
        protected final Good food, labor, money, leisure;

        // This Agent's markets
        protected final Market foodMarket, laborMarket;

        /**
         * Construct a BaseLineAgent.
         */
        public Baseline2Laborer(Economy economy, Map<String, Good> goods) {
            super(economy, goods);

            // Goods
            this.food = goods.get(Goods.Food.toString());
            this.labor = goods.get(Goods.Labor.toString());
            this.money = goods.get(Goods.Money.toString());
            this.leisure = goods.get(Goods.Leisure.toString());

            Errors.AssertHasGoods(this, EnumSet.allOf(Goods.class));

            // Markets
            this.foodMarket = this.economy.getMarketFor(this.money.getName(),
                    this.food.getName());
            this.laborMarket = this.economy.getMarketFor(this.money.getName(),
                    this.labor.getName());
            Errors.Assert(this.foodMarket != null && this.laborMarket != null);
        }

        /**
         * Calls baseLine2LaborerSubclassAct(), THEN converts any unsold labor
         * to leisure. Note that ANY labor which the laborer even tries to sell
         * will not be converted to leisure, assuming that the Market used sets
         * aside all bid labor.
         */
        protected final void ownedLaborerSubclassAct() {
            this.leisure.deposit(convertToLeisure(Laborer.LABOR_ALLOWANCE
                    - this.getLastLaborSold()));

            this.baseLine2LaborerSubclassAct();
            // this.leisure.deposit(convertToLeisure(this.labor
            // .withdraw(this.labor.getQuantity())));
        }

        /**
         * Should be implemented by subclasses to perform further actions when
         * act() is called. Called BEFORE labor is converted to leisure.
         */
        abstract protected void baseLine2LaborerSubclassAct();

        /**
         * Returns a List of food, labor, money, and leisure.
         */
        public Collection<Good> getGoods() {
            Collection<Good> goods = new ArrayList<Good>();
            goods.add(this.food);
            goods.add(this.labor);
            goods.add(this.money);
            goods.add(this.leisure);

            return goods;
        }

        /**
         * Returns food, labor, money, or leisure. Or, throws an exception.
         */
        public Good getGood(String goodName) {
            switch (Goods.valueOf(goodName)) {
            case Money:
                return this.money;
            case Food:
                return this.food;
            case Labor:
                return this.labor;
            case Leisure:
                return this.leisure;
            default:
                throw new RuntimeException("Agent does not have Good "
                        + goodName);
            }
        }

        /**
         * Returns the Agent's quantity of money.
         */
        public double getNetWorth() {
            return this.money.getQuantity();
        }

        /**
         * Get this Agent's food (required by Laborer)
         */
        protected Good getFood() {
            return this.food;
        }

        /**
         * Get this Agent's labor (required by Laborer)
         */
        protected Good getLabor() {
            return this.labor;
        }
    }

    /**
     * Abstract class which sets up the Goods and Markets used in the Baseline2
     * economy.
     * 
     * @author Michael Adelson
     */
    public static abstract class Baseline2Farm extends OwnedFirm {

        // This Agent's goods
        protected final Good food, labor, money;

        // This Agent's markets
        protected final Market foodMarket, laborMarket;

        /**
         * Construct a BaselineFarm
         */
        public Baseline2Farm(Economy economy, Map<String, Good> goods) {
            super(economy, goods);

            // Goods
            this.food = goods.get(Goods.Food.toString());
            this.labor = goods.get(Goods.Labor.toString());
            this.money = goods.get(Goods.Money.toString());
            Errors.AssertHasGoods(this, EnumSet.of(Goods.Food, Goods.Labor,
                    Goods.Money));

            // Markets
            this.foodMarket = this.economy.getMarketFor(this.money.getName(),
                    this.food.getName());
            this.laborMarket = this.economy.getMarketFor(this.money.getName(),
                    this.labor.getName());
            Errors.Assert(this.foodMarket != null && this.laborMarket != null);
        }

        /**
         * The implementation of production. Converts available Labor to Food.
         * 
         * @return the amount produced.
         */
        protected double produce() {
            double amountProduced = convertToFood(labor.withdraw(labor
                    .getQuantity()));
            food.deposit(amountProduced);

            return amountProduced;
        }

        /**
         * This Firm is only Bankrupt when it has no labor, money, or food left.
         * Note that this call is only necessarily valid at the beginning or end
         * of a time step.
         */
        public boolean isBankrupt() {
            return this.money.getQuantity() <= 0.0
                    && this.food.getQuantity() <= 0.0
                    && this.labor.getQuantity() <= 0.0;
        }

        /**
         * Calls baseLine2FarmSubclassAct().
         */
        protected final void ownedFirmSubclassAct() {
            this.baseLine2FarmSubclassAct();
        }

        /**
         * Should be implemented by subclasses to perform further actions during
         * act().
         */
        abstract protected void baseLine2FarmSubclassAct();

        /**
         * Returns a List of food, labor, money.
         */
        public Collection<Good> getGoods() {
            Collection<Good> goods = new ArrayList<Good>();
            goods.add(this.food);
            goods.add(this.labor);
            goods.add(this.money);

            return goods;
        }

        /**
         * No activation costs defined.
         */
        public final Map<String, Double> getRemainingActivationCosts() {
            Errors.err("No activation costs defined");
            return new HashMap<String, Double>();
        }

        /**
         * Returns food, labor, or money, or throws an exception.
         */
        public Good getGood(String goodName) {
            switch (Goods.valueOf(goodName)) {
            case Money:
                return this.money;
            case Food:
                return this.food;
            case Labor:
                return this.labor;
            default:
                throw new RuntimeException("Agent does not have Good "
                        + goodName);
            }
        }

        /**
         * Returns the Agent's quantity of money.
         */
        public double getNetWorth() {
            return this.money.getQuantity();
        }

        /**
         * Get this Agent's labor (required by Firm)
         */
        protected Good getLabor() {
            return this.labor;
        }

    }

    /*------------------------------ Production ------------------------------*/

    /**
     * The production function for Food in BaseLine2: MIN(laborQuantity *
     * LABOR_PRODUCTIVITY, MAX_PRODUCTION).
     */
    public static double convertToFood(double laborQuantity) {
        return (-0.0652 * laborQuantity * laborQuantity) + 4.0652 * laborQuantity;
//        if (laborQuantity > 30)
//            return MAX_PRODUCTION;
//        else return (-0.0667 * laborQuantity * laborQuantity) + (4 * laborQuantity);
        //return Math.min(laborQuantity * LABOR_PRODUCTIVITY, MAX_PRODUCTION);
    }

    /**
     * The conversion from Labor to Leisure (1 to 1)
     */
    public static double convertToLeisure(double laborQuantity) {
        return laborQuantity;
    }

    /*----------------- Constructor ---------------------------*/

    /**
     * Can't be instantiated.
     */
    private Baseline2Model() {
    }
}
