package eos.models;

import eos.framework.Good;

/**
 * DiscreteGood provides a model-level implementation of all methods for a 
 * Good that should be stored in integer quantities. Classes that extend 
 * DiscreteGood need not implement any additional methods in order to be 
 * compatible with Markets.
 * 
 * Author: Michael Adelson 3-29-09
 */
public abstract class DiscreteGood implements Good {

    // epsilon variable used to test whether input double values are integers
    private static final double eps = 1e-5;
    
    // the quantity represented by this Good
    private int quantity;

    /**
     * Construct a DiscreteGood with quantity 0.
     */
    public DiscreteGood() {
        quantity = 0;
    }

    /**
     * Add amount to this good. Throws a RuntimeExceptoin if the difference
     * between amount and Math.round(amount) is > eps, or if amount is negative.
     */
    public void deposit(double amount) {
        int iAmount = (int)Math.round(amount);
        if (iAmount < 0) throw new RuntimeException("Amount must be >= 0");
        if (Math.abs(amount - iAmount) > eps)
            throw new RuntimeException("Labor only accepts integer quantities");
        quantity += iAmount;
    }
    
    /**
     * Return the quantity of this good as a double.
     */
    public double getQuantity() {
        return quantity;
    }

    /**
     * Remove amount of this good. If this DiscreteGoods's quantity is <
     * amount, quantity is set to 0. Returns the amount of good successfully 
     * withdrawn. A RuntimeException will be thrown if amount is negative, or if
     * the difference between Math.round(amount) and amount is > eps.
     */
    public double withdraw(double amount) {
        int iAmount = (int)Math.round(amount);
        if (iAmount < 0) throw new RuntimeException("Amount must be >= 0");
        if (Math.abs(amount - iAmount) > eps)
            throw new RuntimeException("Labor only accepts integer quantities");
        if (iAmount > quantity) {
            int oldQuantity = quantity;
            quantity = 0;
            return oldQuantity;
        }
        quantity -= iAmount;
        return iAmount;
    }
    
    /**
     * Returns true, since this Good is stored in integer quantities.
     */
    public final boolean isDiscrete() { return true;}
}
