/**
 * 
 */
package eos.simulations;

import java.io.PrintWriter;
import java.util.*;
import eos.agents.*;
import eos.economies.BL4321Economy;
import eos.framework.Economy;
import eos.framework.Market;
import eos.io.*;
import eos.io.Simulation.InitializationFunction;
import eos.markets.*;
import eos.models.*;
import eos.models.Baseline4Model.*;
import eos.utilities.Errors;

/**
 * @author Anthony DeLuise, Jr.
 * 
 */
public class BL4Sim1 {

    // parameters
    static int numFarms = 5;
    static int numLaborers = (35/*-5*/) * numFarms;
    //static int numTraders =  5 * numFarms;

    static Simulation sim;

    static {
        sim = new Simulation("BL4-1", BL4321Economy.class,  /*678294307504387L*/ 
        new Random().nextLong());
        sim.addGoods(Baseline4Good.class, "Food", "Labor", "Money", "Utility");

        sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Labor",
                "Utility");
//         sim
//         .addMarkets(BuyerPostedPriceBargainingMarket.class, "Money",
//         "Labor");
//         sim.addMarkets(CallAuctionMarket.class, "Money", "Food", "Utility");

        sim.addAgents(BL4Laborer.class, numLaborers);
        //sim.addAgents(BL3Trader.class, numTraders);
        sim.addOwnersAndFirms(BL4Owner.class, numFarms, BL4Farm.class, 1);
        sim.addOwnersAndFirms(BL4Owner.class, numFarms, BL4Factory.class, 1);
       
        sim.setQuantities(BL4Laborer.class, 33, 0, 60, 0);
        sim.setQuantities(BL4Owner.class, 33, 0, 90, 0);
        sim.setQuantities(BL4Farm.class, 20, 0, 100, 0);
        sim.setQuantities(BL4Factory.class, 0, 0, 100,5);
        
        
        // set the prices before running
        sim.addInitializationFunction(new InitializationFunction() {
            public void initialize(Economy economy) {
                Baseline4Model.HISTORICAL_LABOR_PRICE = (12.0 / 24.0)
                        * Baseline4Model.HISTORICAL_FOOD_PRICE;
            }
        });
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        sim.setDefaultPrintPeriod(50);
        sim.setPerturbation(10);
        // sim.addDataPrinter(new GoodTrackerDataPrinter("Money", false));
        long clock = System.currentTimeMillis();

        if (args == null || args.length == 0 || args[0] == null
                || args[0].length() == 0)
            sim.run(100000);
        else {
            String s = args[0];
            switch (s.toLowerCase().charAt(0)) {
            case 'm':
                multiRunTest();
                break;
            case 's':
                percentStableTest();
                break;
            case 'p':
                performanceTest();
                break;
            case 't':
                traderTest();
                break;
            default:
                System.err.println("Unknown input: " + args[0]);
            }

        }

        System.out.println(System.currentTimeMillis() - clock);
    }

    private static void traderTest() {
        sim.addAgents(BL3Trader.class, 1);
        sim.setQuantities(BL3Trader.class, 330, 0, 60, 0);
        sim.run(30000);
    }

    private static void multiRunTest() {
        String filename = "bl4_conv_test.csv";
        final int runs = 50, steps = 10000;

        //sim.setPerturbation(10);
        sim.disableDefaultCSVPrinter();
        sim.addPrinter(CSVMultiRunCleanupPrinter.class, filename, 0);

        FixedDataDataPrinter pr = new FixedDataDataPrinter("Price Ratio");

        sim.addDataPrinter(pr, filename);

        sim.addDataPrinter(new GoodTrackerDataPrinter("Utility",
                BL4Laborer.class, true), filename);
        sim.addDataPrinter(new GoodTrackerDataPrinter("Utility",
                BL4Owner.class, true), filename);

        FixedDataDataPrinter fp = new FixedDataDataPrinter("Average Food Price");
        FixedDataDataPrinter lp = new FixedDataDataPrinter(
                "Average Labor Price");
        FixedDataDataPrinter up = new FixedDataDataPrinter("Average Utility Price");
        FixedDataDataPrinter fv = new FixedDataDataPrinter(
                "Average Food Volume");
        FixedDataDataPrinter lv = new FixedDataDataPrinter(
                "Average Labor Volume");
        FixedDataDataPrinter uv = new FixedDataDataPrinter(
                "Average Utility Volume");

        sim.addDataPrinter(fp, filename);
        sim.addDataPrinter(lp, filename);
        sim.addDataPrinter(up, filename);
        sim.addDataPrinter(fv, filename);
        sim.addDataPrinter(lv, filename);
        sim.addDataPrinter(uv, filename);


        for (int i = 0; i < runs; i++) {
            BL4321Economy e = sim.getEconomy(BL4321Economy.class);
            Market fm = e.getMarketFor("Money", "Food"), lm = e.getMarketFor(
                    "Money", "Labor"), um = e.getMarketFor("Money", "Utility");

            double fpav = 0, lpav = 0, upav = 0, fvav = 0, lvav = 0, uvav = 0;
            for (int j = 0; j < steps; j++) {
                e.step();
                fpav += fm.getLastMarketPrice();
                lpav += lm.getLastMarketPrice();
                upav += um.getLastMarketPrice();
                fvav += fm.getLastMarketVolume();
                lvav += lm.getLastMarketVolume();
                uvav += um.getLastMarketVolume();
            }
            fp.setData(e, (fpav / (double) steps) + "");
            lp.setData(e, (lpav / (double) steps) + "");
            up.setData(e, (upav / (double) steps) + "");
            fv.setData(e, (fvav / (double) steps) + "");
            lv.setData(e, (lvav / (double) steps) + "");
            uv.setData(e, (uvav / (double) steps) + "");
            pr.setData(e, (lpav / fpav) + "");

            e.cleanupPrinters();
        }

        sim.cleanupPrinters();
    }

    private static void percentStableTest() {
        sim.disablePrinting();
        sim.setPerturbation(0);

        int numAgents = -1;
        final int runs = 100, steps = 10000;
        double numUnstable = 0.0;

        for (int i = 0; i < runs; i++) {
            BL4321Economy e = sim.getEconomy(BL4321Economy.class);
            if (numAgents == -1)
                numAgents = e.getAgents().size();
            e.run(steps);
            Errors.Assert(e.getAgents().size() <= numAgents);
            if (e.getAgents().size() == numAgents)
                System.out.println("Stable with seed "
                        + sim.getRandomSeeds().get(i));
            else {
                System.out.println("Unstable with seed "
                        + sim.getRandomSeeds().get(i));
                numUnstable++;
            }
        }

        System.out.println("Percent Stable = "
                + ((1 - (numUnstable / (double) runs)) * 100));
    }

    private static void performanceTest() {
        sim.disablePrinting();
        final int steps = 10;

        int pop = 4 * numFarms + numLaborers;

        PrintWriter pw = Printers.getPrintWriterFor("performance.csv");
        pw
                .println("Number of Agents, Time Per Step (millis), Time Per Step (nanos)");

        for (int i = 1; i <= 100; i++) {
            BL4321Economy e = sim.getEconomy(BL4321Economy.class);
            Errors.Assert(e.getAgents().size() == (pop * i));

            long mclock = System.currentTimeMillis();
            long nclock = System.nanoTime();
            e.run(steps);
            long ntime = (System.nanoTime() - nclock) / steps;
            long mtime = (System.currentTimeMillis() - mclock) / steps;

            pw.println((pop * i) + ", " + mtime + ", " + ntime);
            System.out.println((pop * i) + ", " + mtime + ", " + ntime);

            sim.addAgents(BL4Laborer.class, numLaborers);
            sim.addOwnersAndFirms(BL4Owner.class, numFarms, BL4Farm.class, 1);
            sim
                    .addOwnersAndFirms(BL4Owner.class, numFarms,
                            BL4Factory.class, 1);
        }

        pw.close();

        System.out.println("Output saved to performance.csv");
    }

}
