package eos.utilities;

import java.util.EnumSet;
import eos.framework.Agent;

/**
 * A utilities class for throwing exceptions and printing errors.
 * 
 * Using the Errors class provides a standard format for exceptions thrown in
 * EOS code. Provides supports for warnings (don't crash the simulation) and
 * errors (stop the simulation immediately).
 * 
 * @author Michael Adelson
 */
public class Errors {

    /**
     * Should Errors print warnings? Disable/Enable with
     * disable/enableWarnings().
     */
    private static boolean printWarnings = true;

    /**
     * Basic error messages used by Errors
     */
    private static final String baseMessage = "\n\nEOS Error:\n\n",
            warningMessage = "\n\nEOS Warning:\n\n",
            warningMessage2 = "\n\n(Disable Warnings with Errors.disableWarnings())";

    /*------------------------- ERR and QUIT ---------------------------------*/

    /**
     * Throw a new RuntimeException with message as the error message.
     */
    public static final void err(String message) {
        throw new RuntimeException(baseMessage + message + "\n");
    }

    /**
     * Throw ex (as a RuntimeException), but first print message. This gives a
     * full stack trace.
     */
    public static final void err(Exception ex, String message) {
        System.err.println(baseMessage + message + "\n");
        if (ex instanceof RuntimeException)
            throw (RuntimeException) ex;
        throw getRuntimeException(ex);
    }

    /**
     * Print message to standard error and exit the program with -1.
     */
    public static final void quit(String message) {
        System.err.println(baseMessage + message);
        System.exit(-1);
    }

    /**
     * @return a RuntimeException "version" of ex. Can be used to throw checked
     *         exceptions as unchecked exceptions.
     */
    public static final RuntimeException getRuntimeException(Exception ex) {
        String message = "";
        if (!(ex instanceof RuntimeException))
            message += "\nOriginal Exception Class\t: "
                    + ex.getClass().getName();
        if (ex.getCause() != null)
            message += "\nCause\t\t\t\t: " + ex.getCause();
        if (ex.getMessage() != null)
            message += "\nMessage\t\t\t\t: " + ex.getMessage();
        message += "\n";
        RuntimeException rex = new RuntimeException(message);
        rex.setStackTrace(ex.getStackTrace());
        return rex;
    }

    /*--------------------------- WARNINGS -----------------------------------*/

    /**
     * Disable prints due to warnings.
     */
    public static final void disableWarnings() {
        printWarnings = false;
    }

    /**
     * Enable prints due to warnings.
     */
    public static final void enableWarnings() {
        printWarnings = true;
    }

    /**
     * Prints message to stderr if warnings are enabled.
     * 
     * @see Errors.AssertWarning()
     */
    public static final void warn(String message) {
        if (printWarnings)
            System.err.println(warningMessage + message + warningMessage2);
    }

    /**
     * Warn with message if problem is true. Does nothing if warnings are
     * disabled.
     */
    public static final void warnIf(boolean problem, String message) {
        if (problem && printWarnings)
            warn(message);
    }

    /*------------------------------ ASSERTS ------------------------------*/

    /**
     * Throw an exeption if claim is false.
     */
    public static final void Assert(boolean claim) {
        if (!claim)
            err("Assertion failed");
    }

    /**
     * Err with message if claim is false.
     */
    public static final void Assert(boolean claim, String message) {
        if (!claim)
            err("Assertion failed: " + message);
    }

    /**
     * Err if obj is null.
     */
    public static final void AssertNonNull(Object obj) {
        if (obj == null)
            err("Assertion failed: object == null");
    }

    /**
     * Err with message if obj is null.
     */
    public static final void AssertNonNull(Object obj, String message) {
        if (obj == null)
            err("Assertion failed: " + message);
    }

    /**
     * Assert that agent has non-null Goods with the names specified by the
     * String array.
     */
    public static final void AssertHasGoods(Agent agent, String[] requiredGoods) {
        for (String goodName : requiredGoods)
            Assert(agent.getGood(goodName) != null, "Agent must have Good "
                    + goodName);
    }

    /**
     * Assert that agent has non-null Goods with the names specified by the
     * EnumSet (the enumeration constants should match the names of the Goods).
     */
    public static final void AssertHasGoods(Agent agent,
            EnumSet<?> requiredGoods) {
        for (Enum<?> e : requiredGoods)
            Assert(agent.getGood(e.toString()) != null, "Agent must have Good "
                    + e.toString());
    }
}
